# -*- coding: utf-8 -*-
"""
Created on Tue May  4 12:22:49 2021

@author: hs-702
"""

import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
#%matplotlib inline

from sklearn.model_selection import train_test_split
from sklearn.metrics import accuracy_score, precision_score, recall_score, roc_auc_score
from sklearn.metrics import f1_score, confusion_matrix, precision_recall_curve, roc_curve
from sklearn.preprocessing import StandardScaler
from sklearn.linear_model import LogisticRegression

diabetes_data = pd.read_csv('C:/Users/hs-702/Desktop/veroro/diabetes.csv')
print(diabetes_data['Outcome'].value_counts())
diabetes_data.head(3)



#Estimator는 LogisticRegressor, train_test_split은 0.2, 156, 정확도, 정밀도, 재현율 구하기
X=diabetes_data.drop('Outcome', axis=1)
y=diabetes_data['Outcome']
                     

X_train, X_test, y_train, y_test=train_test_split(X, y, test_size=0.2, random_state=156)


lr_clf=LogisticRegression()
lr_clf.fit(X_train, y_train)
pred=lr_clf.predict(X_test)

accuracy=accuracy_score(y_test, pred)
precision=precision_score(y_test, pred)
recall=recall_score(y_test, pred)

print(accuracy)
print(precision)
print(recall)


 
#precision_recall_curve 함수이용해서 threshold에 의한 재현율, 정밀도 그래프 그리기
pred_proba1=lr_clf.predict_proba(X_test)[:,1]
precisions, recalls, thresholds = precision_recall_curve(y_test, pred_proba1)
plt.plot(thresholds , precisions[ 0:len(thresholds)] )
plt.plot(thresholds , recalls[ 0:len(thresholds)] )

#'Glucose', 'BloodPressure', 'SkinThickness', 'Insulin', 'BMI'의 각 Column에서 0이 차지하는 비율 구하기
total_count=diabetes_data['Glucose'].count()
zero= ['Glucose', 'BloodPressure', 'SkinThickness', 'Insulin', 'BMI']
for x in zero:
    zero_count=diabetes_data[diabetes_data[x]==0][x].count()
    print('{0} 0건이 차지하는 비율은 {1:.2f}%입니다'.format(x, 100*zero_count/total_count))

    
#각 feature들의 element 중 0의 값을 평균값으로 대체
feature_mean=diabetes_data[zero].mean()
diabetes_data[zero]=diabetes_data[zero].replace(0, feature_mean)


'''
#선생님의 답

pred_proba_c1 = lr_clf.predict_proba(X_test)[:, 1]

precisions, recalls, thresholds = precision_recall_curve( y_test, pred_proba_c1)

print(len(thresholds))

plt.plot(thresholds , precisions[ 0:len(thresholds)] )
plt.plot(thresholds , recalls[ 0:len(thresholds)] )

zero_features = ['Glucose', 'BloodPressure','SkinThickness','Insulin','BMI']

for row in zero_features:
    temp = X[X[row]==0][row]
    print(round((temp.shape[0]/X.shape[0]) * 100 , 2))
    X[row] = X[row].replace(0, X[row].mean())
    
for row in zero_features:
    temp = X[X[row]==0][row]
    print(round((temp.shape[0]/X.shape[0]) * 100 , 2))
'''


# StandardScaler 클래스로 feature 데이트 세트에 스케일링 적용
scaler = StandardScaler( )
X_scaled = scaler.fit_transform(X)
 
# LogisticRegression 이용 학습 및 예측
X_train, X_test, y_train, y_test = train_test_split(X_scaled, y, test_size = 0.2,\
                                                    random_state = 156, stratify=y)
lr_clf = LogisticRegression()
lr_clf.fit(X_train , y_train)
pred = lr_clf.predict(X_test)
pred_proba_c1 = lr_clf.predict_proba(X_test)[:,1]
pred_proba_c1 = pred_proba_c1.reshape(-1,1)
thresholds = [0.3 , 0.33 ,0.36,0.39, 0.42 , 0.45 ,0.48, 0.50]

#Binarizer 를 이용해서 각각의 threshold값에 의한 예측값을 구한다

from sklearn.preprocessing import Binarizer

for row in thresholds:
    binarizer = Binarizer(threshold=row).fit(pred_proba_c1) 
    custom_predict = binarizer.transform(pred_proba_c1)
    print('임곗값:',row)
    confusion = confusion_matrix( y_test, custom_predict)
    accuracy = accuracy_score(y_test , custom_predict)
    precision = precision_score(y_test , custom_predict)
    recall = recall_score(y_test , custom_predict)
    f1 = f1_score(y_test,custom_predict)
    roc_auc = roc_auc_score(y_test, pred_proba_c1)
    temp = (accuracy + precision + recall + f1 + roc_auc) / 5
    print('정확도: {0:.4f}, 정밀도: {1:.4f}, 재현율: {2:.4f},\
    F1: {3:.4f}, AUC:{4:.4f}, 5개평균:{5:.4f}'.format(accuracy,\
        precision, recall, f1, roc_auc, temp))
    
    








